
static char rcsid[] = "$Id: lips_plot.c,v 1.1 1992/12/14 06:01:39 nakano Exp nakano $";

#include    <stdio.h>
#include    "medef.h"

typedef char  lips_marker_ptrn_t[128];

static char     skip_bugs[] = { '\n','\0' };
static char     reset_para[] = { ESC,'=','\0' };
static char     a4_landscape[] = { CSI,'1','5',';',';','p','\0' };
static char     text_dot[] = { CSI,'7',' ','I','\0' };
static char     cancel_auto[] = { CSI,'?','2','h','\0' };
static char     spec_pos[] = { CSI,'1','1','h','\0' };
static char     paint_mode[] = { CSI,'2','&','z','\0' };
static char     vector_mode[] = { CSI,'&','}','\0' };
static char     begin_picture[] = { '#',IS2,'\0' };
static char     scale_mode[] = { '!','0','#','1',IS2,'\0' };
static char     picture_body[] = { '$',IS2,'\0' };
static char     text_mode[] = {'}','p',IS2,CSI,'\0' };

static FILE    *fp;
static char    *filename, *mode;
FILE    *fopen();

static void    put_str();
static void    put_num();
static void    put_digit();
static void    put_pointlist();

/* =============================================================================== */
static void    put_str( str, strlen )
char    *str;
int     strlen;
{
    while( strlen-- > 0 )
        putc( *str++, fp );
}

/* ------------------------------------------------------------------------------- */
static void    put_num( int_data )
int     int_data;
{
    int     c, i, len;
    char    buf[128];

    i = abs( int_data );
    buf[0] = i & 0X0F;
    if( int_data >= 0 )
        buf[0]  |= 0X30;
    else
        buf[0]  |= 0X20;
    len = 1;
    i >>= 4;
    while( i > 0 ) {
        buf[len++] = i & 0X3F | 0X40;
        i >>= 6;
    }

    for( i = 0; i < len/2; i++ ) {
        c            = buf[i];
        buf[i]       = buf[len-1-i];
        buf[len-1-i] = c;
    }
    put_str( buf, len );
}

/* ------------------------------------------------------------------------------- */
static void    put_digit( n )
int     n;
{
    fprintf( fp, "%0d", n );
}

/* ------------------------------------------------------------------------------- */
static void    put_pointlist( n, p )
int     n, (*p)[][2];
{
    int     i, x, y;

    x = 0;
    y = 0;
    for( i = 0; i < n; i++ ) {
        put_num( (*p)[i][0] - x );
        put_num( (*p)[i][1] - y );
        x  = (*p)[i][0];
        y  = (*p)[i][1];
    }
    putc( IS2, fp );
}

/* =============================================================================== */
void    lips_circle( cx, cy, r )
int cx, cy, r;
{                                       
    putc( '5', fp );
    put_num( cx );
    put_num( cy );
    put_num( r );
    putc( IS2, fp );
}
                  
/* ------------------------------------------------------------------------------- */
void    lips_clip( lx, ly, rx, ry )
int     lx, ly, rx, ry;
{
    putc( ')', fp );
    put_num( lx );
    put_num( rx );
    put_num( ly );
    put_num( ry );
    putc( IS2, fp );
    put_str( "*0\036", 3 );
}

/* ------------------------------------------------------------------------------- */
void    lips_fill_area( x, y )
int     x, y;
{
    put_str( "}9", 2 );
    put_num( x );
    put_num( y );
    putc( IS2, fp );
}

/* ------------------------------------------------------------------------------- */
void    lips_init( fname )
char	*fname;
{
    fp = fopen( fname, "w" );
    put_str( skip_bugs, strlen( skip_bugs ) );
    put_str( reset_para, strlen( reset_para ) );
    put_str( a4_landscape, strlen( a4_landscape ) );
    put_str( text_dot, strlen( text_dot ) );
    put_str( cancel_auto, strlen( cancel_auto ) );
    put_str( spec_pos, strlen( spec_pos ) );
    put_str( paint_mode, strlen( paint_mode ) );
    put_str( vector_mode, strlen( vector_mode ) );
    put_str( begin_picture, strlen( begin_picture ) );
    put_str( scale_mode, strlen( scale_mode ) );
    put_str( picture_body, strlen( picture_body ) );

    putc( '(', fp );
    put_num( 0 );
    put_num( 0 );
    put_num( XSIZE );
    put_num( YSIZE );
    putc( IS2, fp );               /* vdc extent */
}

/* ------------------------------------------------------------------------------- */
void    lips_line_type( t )
int     t;
{
    put_str( "E1", 2 );
    put_num( t );
    putc( IS2, fp );
}

/* ------------------------------------------------------------------------------- */
void    lips_line_width( w )
int     w;
{
    put_str( "F1", 2 );
    put_num( w );
    putc( IS2, fp );
}

/* ------------------------------------------------------------------------------- */
void    lips_markerdef( m, ptn )
int     m;
lips_marker_ptrn_t  ptn;
{
    int     i, patternlen;
    char    pattern[128];

    if( m != -10 && m != -11 ) return;
    for( i = 0; i < 128; i++ )
        pattern[i] = ptn[i];
    patternlen = 128;                              
    put_str( "}@1", 3 );
    put_num( m );
    putc( IS2, fp );
    put_str( pattern, patternlen );
}
/* ------------------------------------------------------------------------------- */
void    lips_markersize( s )
int     s;
{
    putc( 'B', fp );
    put_num( s );
    putc( IS2, fp );
}

/* ------------------------------------------------------------------------------- */
void    lips_polygon( n, p )
int     n;
int     (*p)[][2];
{
    if( n <= 0 || n > 360 ) return;
    putc( '2', fp );
    put_pointlist( n, p );
}

/* ------------------------------------------------------------------------------- */
void    lips_polyline( n, p )               
int     n;
int     (*p)[][2];
{                                           
    if( n > 360 ) {  
        lips_polyline( 360, p );
        lips_polyline( n - 359, (int(*)[][2])((*p)[359]) );
    } else {                                 
        if( n <= 0 ) return;                 
        putc( '1', fp );
        put_pointlist( n, p );
    }
}
     
/* ------------------------------------------------------------------------------- */
void    lips_polymarker( n, p )
int     n;
int     (*p)[][2];
{
    if( n > 360 ) {
        lips_polymarker( 360, p );
        lips_polymarker( n - 359, (int(*)[][2])(*p)[359] );
    } else {                                 
        if( n <= 0 ) return;                 
        putc( '0', fp );
        put_pointlist( n, p );
    }
}
                   
/* ------------------------------------------------------------------------------- */
void    lips_set_fptn( p )
int     p;
{
    putc( 'I', fp );
    put_num( p );
    putc( IS2, fp );
}

/* ------------------------------------------------------------------------------- */
void    lips_set_marker( m )
int     m;
{
    putc( 'A', fp );
    put_num( m );
    putc( IS2, fp );
}
              
/* ------------------------------------------------------------------------------- */
void    lips_text( x, y, str )
int     x, y;
char    str[256];
{
        put_str( text_mode, strlen( text_mode ) );
        put_digit( YSIZE - y );
        putc( ';', fp );
        put_digit( x );
        putc( 'f', fp );
        put_str( str, min(256, strlen(str)) );
        putc( CSI, fp );
        put_str( "&}", 2 );
}

/* ------------------------------------------------------------------------------- */
void    lips_formfeed( )       /* new page */
{
    putc( FF, fp );
}

/* ------------------------------------------------------------------------------- */
void    lips_terminate( )
{
    putc( FF, fp );
    putc( '%', fp );
    putc( IS2, fp );
    put_str( "}p", 2 );
    putc( IS2, fp );
}

