
static char rcsid[] = "$Id: g_dev_m.c,v 1.1 1992/12/14 06:01:23 nakano Exp nakano $";

#ifdef __STDC__
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif

#include "g_private.h"
#include "g_dev_m.h"

#define M_CHECK_POS(pos, proc) \
{ if (g_m_check_pos (pos, proc)) return; }

static void
g_m_put_title (fp)
    FILE *	fp;
{
    static char	title[M_FILE_TITLE_LEN] = M_FILE_TITLE;
    int		i;

    for (i = 0; i < M_FILE_TITLE_LEN; i ++)
    {
	M_PUT_CHAR (fp, title[i]);
    }
}
 
static void
g_m_put_time (fp)
    FILE *	fp;
{
    static char	stamp[M_FILE_TIME_LEN];
    time_t	t;
    int		i;

    (void) time (& t);
    (void) strncpy (stamp, ctime (& t), M_FILE_TIME_LEN);

    for (i = 0; i < M_FILE_TIME_LEN; i ++)
    {
	M_PUT_CHAR (fp, stamp[i] == '\n' ? '\0' : stamp[i]);
    }
}
 
static void
g_m_put_pos (fp, pos)
    FILE *	fp;
    int		pos;
{
    M_PUT_BYTE_2 (fp, pos);
}
 
static void
g_m_put_line_color (fp, color)
    FILE *	fp;
    G_COLOR	color;
{
    M_PUT_HEADER (fp, M_LINE_COLOR, 0);
    M_PUT_COLOR (fp, color);
}
 
static void
g_m_put_line_width (fp, width)
    FILE *		fp;
    G_LINE_WIDTH	width;
{
    M_PUT_HEADER (fp, M_LINE_WIDTH, 0);
    M_PUT_LINE_WIDTH (fp, width);
}
 
static void
g_m_put_line_type (fp, type)
    FILE *	fp;
    G_LINE_TYPE	type;
{
    M_PUT_HEADER (fp, M_LINE_TYPE, 0);
    M_PUT_LINE_TYPE (fp, type);
}
 
static void
g_m_put_area_color (fp, color)
    FILE *	fp;
    G_COLOR	color;
{
    M_PUT_HEADER (fp, M_AREA_COLOR, 0);
    M_PUT_COLOR (fp, color);
}
 
static void
g_m_put_text_color (fp, color)
    FILE *	fp;
    G_COLOR	color;
{
    M_PUT_HEADER (fp, M_TEXT_COLOR, 0);
    M_PUT_COLOR (fp, color);
}
 
/*ARGSUSED*/
static void
g_m_put_text_font (fp, font)
    FILE *	fp;
    G_FONT	font;
{
/* SUSPENDED */
}
 
static void
g_m_put_cls (fp)
    FILE *	fp;
{
    M_PUT_HEADER (fp, M_CLS, 0);
}
 
static void
g_m_put_polyline (fp, points, n_points)
    FILE *	fp;
    G_POINT	points[];
    int		n_points;
{
    int		i;

    M_PUT_HEADER (fp, M_POLYLINE, 0);

    if (n_points == 1)
    {
	M_PUT_COUNT (fp, n_points + 1);
	g_m_put_pos (fp, M_TRANSLATE_X (dev, points[0].x));
	g_m_put_pos (fp, M_TRANSLATE_Y (dev, points[0].y));
    }
    else
    {
	M_PUT_COUNT (fp, n_points);
    }

    for (i = 0; i < n_points; i ++)
    {
	g_m_put_pos (fp, M_TRANSLATE_X (dev, points[i].x));
	g_m_put_pos (fp, M_TRANSLATE_Y (dev, points[i].y));
    }

}
 
/*ARGSUSED*/
static void
g_m_put_polygon (fp, points, n_points, edge, fill)
    FILE *	fp;
    G_POINT	points[];
    int		n_points;
    G_BOOL	edge;
    G_BOOL	fill;
{
    int		i;

    /* SUSPENDED : should respect 'edge' */
    M_PUT_HEADER (fp, M_POLYGON, fill ? 1 : 0);
    M_PUT_COUNT (fp, n_points);

    for (i = 0; i < n_points; i ++)
    {
	g_m_put_pos (fp, M_TRANSLATE_X (dev, points[i].x));
	g_m_put_pos (fp, M_TRANSLATE_Y (dev, points[i].y));
    }
}
 
/*ARGSUSED*/
static void
g_m_put_circle (fp, x, y, radius_x, edge, fill)
    FILE *	fp;
    G_POS	x, y;
    G_DIM	radius_x;
    G_BOOL	edge;
    G_BOOL	fill;
{
    /* SUSPENDED : should respect 'edge' */
    M_PUT_HEADER (fp, M_CIRCLE, fill ? 1 : 0);
    g_m_put_pos (fp, M_TRANSLATE_X (dev, x));
    g_m_put_pos (fp, M_TRANSLATE_Y (dev, y));
    g_m_put_pos (fp, M_TRANSLATE_W (dev, radius_x));
}
 
static void
g_m_put_text (fp, x, y, string)
    FILE *	fp;
    G_P_POS	x, y;
    char *	string;
{
    char *	p;

    M_PUT_HEADER (fp, M_TEXT, 0);
    g_m_put_pos (fp, M_TRANSLATE_X (dev, x));
    g_m_put_pos (fp, M_TRANSLATE_Y (dev, y));

    for (p = string; * p; p ++)
    {
	M_PUT_CHAR (fp, * p);
    }

    if ((p - string) % 2 == 0)
    {
	M_PUT_CHAR (fp, '\0');
    }

    M_PUT_CHAR (fp, '\0');
}
 
static G_BOOL
g_m_check_pos (pos, proc)
    G_P_POS	pos;
    char *	proc;
{
    if (pos < -0.5 || 655.355 <= pos)
    {
	char	message[200];

	sprintf (message, "position \"%f\" : out of range", pos);
	g_warning (proc, message);
	return G_YES;
    }

    return G_NO;
}

static void
g_m_flush (dev)
    DEVICE *	dev;
{
    M_PRIV *		priv = (M_PRIV *) dev -> dev_private;

    if (priv -> plotted)
    {
	G_POINT	p;

	p = priv -> points[priv -> n_points - 1];
	g_m_put_polyline (priv -> fp, priv -> points, priv -> n_points);
	priv -> points[0] = p;
	priv -> n_points = 1;
	priv -> plotted = G_NO;
    }
}

static G_BOOL
g_m_initialize (dev)
    DEVICE *	dev;
{
    M_PRIV *		priv = (M_PRIV *) dev -> dev_private;
    extern char *	getenv ();

    priv -> file_name = (g_context.file_name[0] != '\0')
			     ? g_context.file_name : getenv ("METAFILE");

    if (priv -> file_name == (char *) NULL)
    {
	g_warning ("g_device", "no METAFILE environment");
	return G_YES;
    }

    priv -> fp = fopen (priv -> file_name, "w");

    if (priv -> fp == (FILE *) NULL)
    {
	char	message[200];

	sprintf (message, "can not open meta file \"%s\"",  priv -> file_name);
	g_warning ("g_device", message);
	return G_YES;
    }

    g_m_put_title (priv -> fp);
    g_m_put_time (priv -> fp);

    priv -> line.defined = G_NO;
    priv -> area.defined = G_NO;
    priv -> text.defined = G_NO;
    priv -> points[0].x = priv -> points[0].y = 0.0;
    priv -> n_points = 1;
    priv -> plotted = G_NO;
    dev -> x_scale = dev -> y_scale = 1.0;
    dev -> initialize = (G_BOOL (*) G_ARGS ((DEVICE *))) NULL;

    return G_NO;
}

static void
g_m_terminate (dev)
    DEVICE *	dev;
{
    M_PRIV *		priv = (M_PRIV *) dev -> dev_private;

    g_m_flush (dev);

    if (priv -> fp)
	fclose (priv -> fp);

    priv -> fp = (FILE *) NULL;
    dev -> initialize = g_m_initialize;
}

static void
g_m_validate_line (dev)
    DEVICE *	dev;
{
    M_PRIV *	priv = (M_PRIV *) dev -> dev_private;
    FILE *	fp = priv -> fp;
    LINE_PROP *	prop = CURRENT (line);

    g_m_flush (dev);

    if (! priv -> line.defined || priv -> line.color != prop -> color)
	g_m_put_line_color (fp, prop -> color);

    if (! priv -> line.defined || priv -> line.type != prop -> type)
	g_m_put_line_type (fp, prop -> type);

    if (! priv -> line.defined || priv -> line.width != prop -> width)
	g_m_put_line_width (fp, prop -> width);

    priv -> line = * prop;
}

static void
g_m_validate_area (dev)
    DEVICE *	dev;
{
    M_PRIV *	priv = (M_PRIV *) dev -> dev_private;
    AREA_PROP *	prop = CURRENT (area);

    if (! priv -> area.defined || priv -> area.color != prop -> color)
	g_m_put_area_color (priv -> fp, prop -> color);

    priv -> area = * prop;
}

static void
g_m_validate_text (dev)
    DEVICE *	dev;
{
    M_PRIV *	priv = (M_PRIV *) dev -> dev_private;
    FILE *	fp = priv -> fp;
    TEXT_PROP *	prop = CURRENT (text);

    if (! priv -> text.defined || priv -> text.color != prop -> color)
	g_m_put_text_color (fp, prop -> color);

    if (! priv -> text.defined || priv -> text.font != prop -> font)
	g_m_put_text_font (fp, prop -> font);

    priv -> text = * prop;
}

static void
g_m_clear (dev)
    DEVICE *	dev;
{
    g_m_flush (dev);
    g_m_put_cls (((M_PRIV *) dev -> dev_private) -> fp);
}

static void
g_m_line (dev, packet)
    DEVICE *	dev;
    OPAQUE	packet;
{
    M_PRIV *		priv = (M_PRIV *) dev -> dev_private;
    G_LINE_DEF *	p = (G_LINE_DEF *) packet;
    G_POINT *		old_point = & priv -> points[priv -> n_points - 1];

    if (p -> start.x != old_point -> x || p -> start.y != old_point -> y)
    {
	g_m_flush (dev);
	M_CHECK_POS (p -> start.x, "g_plot");
	M_CHECK_POS (p -> start.y, "g_plot");
	priv -> points[0] = p -> start;
    }

    M_CHECK_POS (p -> end.x, "g_plot");
    M_CHECK_POS (p -> end.y, "g_plot");

    if (priv -> n_points >= G_MAX_N_POINTS)
	g_m_flush (dev);

    if (priv -> n_points == 0 ||
	p -> start.x != p -> end.x || p -> start.y != p -> end.y)
    {
	priv -> points[priv -> n_points] = p -> end;
	priv -> n_points ++;
    }

    priv -> plotted = G_YES;
}

static void
g_m_polyline (dev, packet)
    DEVICE *	dev;
    OPAQUE	packet;
{
    G_POLYLINE_DEF *	p = (G_POLYLINE_DEF *) packet;
    int			i;

    for (i = 0; i < p -> n_points; i ++)
    {
	M_CHECK_POS (p -> points[i].x, "g_polyline");
	M_CHECK_POS (p -> points[i].y, "g_polyline");
    }

    g_m_flush (dev);
    g_m_put_polyline (((M_PRIV *) dev -> dev_private) -> fp,
		      p -> points, p -> n_points);
}

static void
g_m_polygon (dev, packet)
    DEVICE *	dev;
    OPAQUE	packet;
{
    G_POLYGON_DEF *	p = (G_POLYGON_DEF *) packet;
    int			i;

    for (i = 0; i < p -> n_points; i ++)
    {
	M_CHECK_POS (p -> points[i].x, "g_polygon");
	M_CHECK_POS (p -> points[i].y, "g_polygon");
    }

    g_m_flush (dev);
    g_m_put_polygon (((M_PRIV *) dev -> dev_private) -> fp,
		     p -> points, p -> n_points, p -> edge, p -> fill);
}

static void
g_m_circle (dev, packet)
    DEVICE *	dev;
    OPAQUE	packet;
{
    G_CIRCLE_DEF *	p = (G_CIRCLE_DEF *) packet;

    M_CHECK_POS (p -> x, "g_circle");
    M_CHECK_POS (p -> y, "g_circle");
    M_CHECK_POS (p -> radius_x, "g_circle");

    g_m_flush (dev);
    g_m_put_circle (((M_PRIV *) dev -> dev_private) -> fp,
		    p -> x, p -> y, p -> radius_x, p -> edge, p -> fill);
}

static void
g_m_text (dev, packet)
    DEVICE *	dev;
    OPAQUE	packet;
{
    G_TEXT_DEF *	p = (G_TEXT_DEF *) packet;

    M_CHECK_POS (p -> x, "g_text");
    M_CHECK_POS (p -> y, "g_text");

    g_m_flush (dev);
    g_m_put_text (((M_PRIV *) dev -> dev_private) -> fp,
		  p -> x, p -> y, p -> string);
}

static M_PRIV		g_m_dev_private;

DEVICE	g_m_device =
{
    /* defined			*/	G_YES,
    /* x_scale			*/	1.0,
    /* y_scale			*/	1.0,
    /* initialize		*/	g_m_initialize,
    /* terminate		*/	g_m_terminate,
    /* validate_bw_color	*/	(PROC *) NULL,
    /* validate_line		*/	g_m_validate_line,
    /* validate_area		*/	g_m_validate_area,
    /* validate_text		*/	g_m_validate_text,
    /* clear			*/	g_m_clear,
    /* line			*/	g_m_line,
    /* polyline			*/	g_m_polyline,
    /* polygon			*/	g_m_polygon,
    /* circle			*/	g_m_circle,
    /* text			*/	g_m_text,
    /* sleep			*/	(PROC *) NULL,
    /* dev_private		*/	(OPAQUE) & g_m_dev_private,
};

