
static char rcsid[] = "$Id: g_init.c,v 1.5 1995/09/27 03:48:23 nakano Exp nakano $";

#include "g_private.h"

/*-------------------------------------------------------------------*
			Private Definitions
 *-------------------------------------------------------------------*/

static G_BOOL	g_initialized = G_NO;

static void
g_scale_init ()
{
    INIT_SLOTS (SCALE_CONTEXT, scale, DEF_N_SCALES);

    scale -> current.defined = G_YES;
    scale -> current.x_offset = 0.0;
    scale -> current.y_offset = 0.0;
    scale -> current.x_scale = 1.0;
    scale -> current.y_scale = 1.0;
}

static void
g_line_init ()
{
    INIT_SLOTS (LINE_CONTEXT, line, DEF_N_LINE_PROPS);

    line -> current.defined = G_YES;
    line -> current.color = DEF_LINE_COLOR;
    line -> current.width = DEF_LINE_WIDTH;
    line -> current.type = DEF_LINE_TYPE;
}

static void
g_area_init ()
{
    INIT_SLOTS (AREA_CONTEXT, area, DEF_N_AREA_PROPS);

    area -> current.defined = G_YES;
    area -> current.color = DEF_AREA_COLOR;
}

static void
g_marker_init ()
{
    INIT_SLOTS (MARKER_CONTEXT, marker, DEF_N_MARKER_PROPS);

    marker -> current.defined = G_YES;
    marker -> current.color = DEF_MARKER_COLOR;
    marker -> current.size = DEF_MARKER_SIZE;
    marker -> current.type = DEF_MARKER_TYPE;
}

static void
g_text_init ()
{
    INIT_SLOTS (TEXT_CONTEXT, text, DEF_N_TEXT_PROPS);

    text -> current.defined = G_YES;
    text -> current.color = DEF_TEXT_COLOR;
    text -> current.font = DEF_TEXT_FONT;
}

/*-------------------------------------------------------------------*
			Public Definitions
 *-------------------------------------------------------------------*/

G_CONTEXT	g_context;

G_COLOR
g_check_color (color, def, proc)
    G_COLOR	color;
    G_COLOR	def;
    char *	proc;
{
    char	message[200];

    if (G_COLOR_MIN <= color && color <= G_COLOR_MAX)
	return color;

    sprintf (message, "invalid color %d treated as %d", color, def);
    g_warning (proc, message);

    return def;
}

G_LINE_WIDTH
g_check_line_width (width, def, proc)
    G_LINE_WIDTH	width;
    G_LINE_WIDTH	def;
    char *		proc;
{
    char	message[200];

    if (width >= 0)
	return width;

    sprintf (message, "invalid line width %d treated as %d", width, def);
    g_warning (proc, message);

    return def;
}

G_LINE_TYPE
g_check_line_type (type, def, proc)
    G_LINE_TYPE	type;
    G_LINE_TYPE	def;
    char *	proc;
{
    char	message[200];

    if (G_LINE_TYPE_MIN <= type && type <= G_LINE_TYPE_MAX)
	return type;

    sprintf (message, "invalid line type %d treated as %d", type, def);
    g_warning (proc, message);

    return def;
}

G_MARKER_SIZE
g_check_marker_size (size, def, proc)
    G_MARKER_SIZE	size;
    G_MARKER_SIZE	def;
    char *		proc;
{
    char	message[200];

    if (size >= 0)
	return size;

    sprintf (message, "invalid marker size %d treated as %d", size, def);
    g_warning (proc, message);

    return def;
}

G_MARKER_TYPE
g_check_marker_type (type, def, proc)
    G_MARKER_TYPE	type;
    G_MARKER_TYPE	def;
    char *		proc;
{
    char	message[200];

    if (G_MARKER_TYPE_MIN <= type && type <= G_MARKER_TYPE_MAX)
	return type;

    sprintf (message, "invalid marker type %d treated as %d", type, def);
    g_warning (proc, message);

    return def;
}

G_FONT
g_check_font (font, def, proc)
    G_FONT	font;
    G_FONT	def;
    char *	proc;
{
    char	message[200];

    if (G_FONT_MIN <= font && font <= G_FONT_MAX)
	return font;

    sprintf (message, "invalid font %d treated as %d", font, def);
    g_warning (proc, message);

    return def;
}

void
g_set_line (color, width, type)
    G_COLOR		color;
    G_LINE_WIDTH	width;
    G_LINE_TYPE		type;
{
    LINE_CONTEXT *	line = g_context.line;

    line -> current.color = color;
    line -> current.width = width;
    line -> current.type = type;
    CALL_PROC (validate_line, NULL);
}

void
g_set_area (color)
    G_COLOR	color;
{
    AREA_CONTEXT *	area = g_context.area;

    area -> current.color = color;
    CALL_PROC (validate_area, NULL);
}

void
g_set_scale (x_left, x_right, y_bottom, y_top,
	     x_left_std, y_top_std, width_std, height_std)
    G_P_POS	x_left;
    G_P_POS	x_right;
    G_P_POS	y_bottom;
    G_P_POS	y_top;
    G_P_POS	x_left_std;
    G_P_POS	y_top_std;
    G_P_DIM	width_std;
    G_P_DIM	height_std;
{
    G_POS		xs, ys;
    SCALE_CONTEXT *	scale = g_context.scale;

    scale -> current.defined = G_YES;
    scale -> current.x_scale = xs = width_std / (x_right - x_left);
    scale -> current.y_scale = ys = height_std / (y_bottom - y_top);
    scale -> current.x_offset = x_left_std - x_left * xs;
    scale -> current.y_offset = y_top_std - y_top * ys;
}

unsigned long
g_set_bw_color (mask)
    unsigned long	mask;
{
    unsigned long	old_mask = g_context.bw_color_mask;

    if (old_mask != mask)
    {
	g_context.bw_color_mask = mask;
	CALL_PROC (validate_bw_color, NULL);
    }

    return old_mask;
}

char *
g_malloc (size)
    unsigned 	size;
{
    char *	p = malloc (size);

    if (p)
	return p;

    g_error ("g_malloc : no memory");
    /*NOTREACHED*/
}

char *
g_realloc (p, size)
    char * 	p;
    unsigned 	size;
{
    p = realloc (p, size);

    if (p)
	return p;

    g_error ("g_realloc : no memory");
    /*NOTREACHED*/
}

void
g_error (message)
    char *	message;
{
    fprintf (stderr, "fatal error : %s\n", message);
    exit (-1);
}

void
g_warning (proc, message)
    char *	proc;
    char *	message;
{
    fprintf (stderr, "%s : %s\n", proc, message);
}

void
g_check_init ()
{
    if (! g_initialized)
	g_error ("g_init : not called");
}

void
Cg_init (file_name, win_width, win_height)
    char *	file_name;
    G_P_DIM	win_width;
    G_P_DIM	win_height;
{               
    if (file_name)
	strncpy (g_context.file_name, file_name, G_MAX_TEXT_LENGTH);
    else
	g_context.file_name[0] = '\0';

    g_context.file_name[G_MAX_TEXT_LENGTH] = '\0';

    if ((int)strlen (g_context.file_name) >= G_MAX_TEXT_LENGTH
	&& file_name[G_MAX_TEXT_LENGTH] != '\0')
    {
	char message[100 + G_MAX_TEXT_LENGTH];

	sprintf (message, "too long file name truncated to \"%s\".\n",
		 g_context.file_name);
	g_warning ("g_init", message);
    }

    g_initialized = G_YES;
    g_context.win_width = win_width;
    g_context.win_height = win_height;
    g_context.frame = 0;
    g_context.bw_color_mask = ~ (1 << G_WHITE);
    g_context.point.x = g_context.point.y = 0;

    g_scale_init ();
    g_line_init ();
    g_area_init ();
    g_marker_init ();
    g_text_init ();
    g_device_init ();
}

void
F77g_init (file_name, name_length, win_width, win_height)
    char *	file_name;
    int *	name_length;
    G_DIM *	win_width;
    G_DIM *	win_height;
{               
    char	name[G_MAX_TEXT_LENGTH + 1];
    int		length = MIN (G_MAX_TEXT_LENGTH, * name_length);

    strncpy (name, file_name, length);
    name[length] = '\0';
    Cg_init (name, * win_width, * win_height);
}

void
g_init__ (file_name, name_length, win_width, win_height)
    char *	file_name;
    int *	name_length;
    G_DIM *	win_width;
    G_DIM *	win_height;
{               
    char	name[G_MAX_TEXT_LENGTH + 1];
    int		length = MIN (G_MAX_TEXT_LENGTH, * name_length);

    strncpy (name, file_name, length);
    name[length] = '\0';
    Cg_init (name, * win_width, * win_height);
}

void
Cg_term ()
{
    g_check_init ();
    g_device_term ();
}

void
F77g_term ()
{
    g_check_init ();
    g_device_term ();
}

void
g_term__ ()
{
    g_check_init ();
    g_device_term ();
}

