
/*
 *	$Id: g_private.h,v 1.4 1995/05/23 07:07:28 nakano Exp nakano $
 */

#include <stdio.h>

#define G_PRIVATE_H

#ifdef __STDC__
#include <stdlib.h>
#include <string.h>
#else
extern char *		malloc ();
extern char *		realloc ();
extern void		exit ();
extern char *		strncpy ();
extern char *		strcpy ();
#endif

#include "glsc.h"
#include "g_config.h"
#include "g_rename.h"

#define DEF_N_DEVICES		2
#define DEF_N_SCALES		4
#define	DEF_N_LINE_PROPS	4
#define	DEF_N_AREA_PROPS	4
#define DEF_N_MARKER_PROPS	4
#define DEF_N_TEXT_PROPS	4

typedef void	PROC ();
typedef char *	OPAQUE;

#define SOME	1
#define MIN(x, y)	((x) < (y) ? (x) : (y))
#define MAX(x, y)	((x) > (y) ? (x) : (y))

#define MEM_OFFSET(type, mem) \
	((char *) & (((type *) & g_context) -> mem) - (char *) & g_context)

#define G_MAX_N_DEVICES	10

typedef struct DEVICE
{
    G_BOOL		defined;
    G_REAL		x_scale;
    G_REAL		y_scale;
    G_BOOL		(* initialize) G_ARGS ((struct DEVICE * dev));
    PROC *		terminate;
    PROC *		validate_bw_color;
    PROC *		validate_line;
    PROC *		validate_area;
    PROC *		validate_text;
    PROC *		clear;
    PROC *		line;
    PROC *		polyline;
    PROC *		polygon;
    PROC *		circle;
    PROC *		text;
    PROC *		sleep;
    OPAQUE		dev_private;	/* private data for the device */
}
DEVICE;

typedef struct
{
    G_DEV_MASK		mask;
    int			n_slots;
    DEVICE		slots[SOME];
}
DEVICE_CONTEXT;

typedef struct
{
    G_BOOL	defined;
    G_POS	x_offset;
    G_POS	y_offset;
    G_POS	x_scale;
    G_POS	y_scale;
}
SCALE;

typedef struct
{
    SCALE		current;
    int			n_slots;
    SCALE		slots[SOME];
}
SCALE_CONTEXT;

typedef struct
{
    G_BOOL		defined;
    G_COLOR		color;
    G_LINE_WIDTH	width;
    G_LINE_TYPE		type;
}
LINE_PROP;

#define DEF_LINE_COLOR	G_BLACK
#define DEF_LINE_WIDTH	0
#define DEF_LINE_TYPE	G_LINE_SOLID

typedef struct
{
    LINE_PROP		current;
    int			n_slots;
    LINE_PROP		slots[SOME];
}
LINE_CONTEXT;

typedef struct
{
    G_BOOL		defined;
    G_COLOR		color;
}
AREA_PROP;

#define DEF_AREA_COLOR	G_WHITE

typedef struct
{
    AREA_PROP		current;
    int			n_slots;
    AREA_PROP		slots[SOME];
}
AREA_CONTEXT;

typedef struct
{
    G_BOOL		defined;
    G_COLOR		color;
    G_MARKER_SIZE	size;
    G_MARKER_TYPE	type;
}
MARKER_PROP;

#define DEF_MARKER_COLOR	G_BLACK
#define DEF_MARKER_SIZE		5
#define DEF_MARKER_TYPE		G_MARKER_STAR

typedef struct
{
    MARKER_PROP		current;
    int			n_slots;
    MARKER_PROP		slots[SOME];
}
MARKER_CONTEXT;

typedef struct
{
    G_BOOL		defined;
    G_COLOR		color;
    G_FONT		font;
}
TEXT_PROP;

#define DEF_TEXT_COLOR	G_BLACK
#define DEF_TEXT_FONT	G_FONT_TIMES_18

typedef struct
{
    TEXT_PROP		current;
    int			n_slots;
    TEXT_PROP		slots[SOME];
}
TEXT_CONTEXT;

typedef struct
{
    double		dummy;	/* for strict alignment */
    char		file_name[G_MAX_TEXT_LENGTH + 1];
    G_DIM		win_width;
    G_DIM		win_height;
    int			frame;
    unsigned long	bw_color_mask;
    G_POINT		point;
    DEVICE_CONTEXT *	device;
    SCALE_CONTEXT *	scale;
    LINE_CONTEXT *	line;
    AREA_CONTEXT *	area;
    MARKER_CONTEXT *	marker;
    TEXT_CONTEXT *	text;
}
G_CONTEXT;

#define CURRENT(name)	(& (g_context.name -> current))

#define EXTEND_SLOTS(type, name, new, max_n_slots, message) \
    type *	name; \
    int		tmp_n = (new) + 1; \
    int		tmp_i; \
 \
    g_check_init (); \
 \
    if (tmp_n > g_context.name -> n_slots) \
    { \
	tmp_n = MAX ((g_context.name -> n_slots * 3 + 1) / 2, tmp_n); \
 \
	if (tmp_n > (max_n_slots)) \
	    g_error (message); \
 \
	name = (type *) g_realloc ((char *) g_context.name, \
	       (unsigned) (sizeof (char) * MEM_OFFSET (type, slots[tmp_n]))); \
 \
	for (tmp_i = name -> n_slots; tmp_i < tmp_n; tmp_i ++) \
	    name -> slots[tmp_i].defined = G_NO; \
 \
	name -> n_slots = tmp_n; \
	g_context.name = name; \
    } \
    else \
    { \
	name = g_context.name; \
    }

#define INIT_SLOTS(type, name, def_n_slots) \
    type *	name; \
    int		tmp_n = MAX (1, def_n_slots);  \
    int		tmp_i; \
 \
    name = (type *) g_malloc ((unsigned) \
		(sizeof (char) * MEM_OFFSET (type, slots[tmp_n]))); \
    name -> n_slots = tmp_n;  \
    g_context.name = name; \
 \
    for (tmp_i = 0; tmp_i < tmp_n; tmp_i ++) \
	name -> slots[tmp_i].defined = G_NO;

#define TRANSLATE_X(s, x)	((x) * (s) -> x_scale + (s) -> x_offset)
#define TRANSLATE_Y(s, y)	((y) * (s) -> y_scale + (s) -> y_offset)
#define TRANSLATE_W(s, x)	((x) * (s) -> x_scale)
#define TRANSLATE_H(s, y)	((y) * (s) -> y_scale)

#define CALL_PROC(proc, packet) \
    (g_device_call (MEM_OFFSET (DEVICE, proc), (OPAQUE) (packet)))

typedef struct
{
    G_POINT	start;
    G_POINT	end;
}
G_LINE_DEF;

typedef struct
{
    int		n_points;
    G_POINT	points[G_MAX_N_POINTS];
}
G_POLYLINE_DEF;

typedef struct
{
    int		n_points;
    G_POINT	points[G_MAX_N_POINTS];
    G_BOOL	edge;
    G_BOOL	fill;
}
G_POLYGON_DEF;

typedef struct
{
    G_POS	x, y;
    G_DIM	radius_x;
    G_BOOL	edge;
    G_BOOL	fill;
}
G_CIRCLE_DEF;

typedef struct
{
    G_POS	x, y;
    char *	string;
}
G_TEXT_DEF;

typedef struct
{
    G_REAL	time;
}
G_SLEEP_DEF;

extern G_CONTEXT	g_context;
extern void		g_device_call G_ARGS ((int offset, OPAQUE packet));
extern void		g_device_init G_ARGS ((void));
extern void		g_device_term G_ARGS ((void));
extern G_COLOR		g_check_color G_ARGS
			    ((G_COLOR color, G_COLOR def, char * proc));
extern G_LINE_WIDTH	g_check_line_width G_ARGS ((G_LINE_WIDTH width,
						    G_LINE_WIDTH def,
						    char * proc));
extern G_LINE_TYPE	g_check_line_type G_ARGS ((G_LINE_TYPE type,
						   G_LINE_TYPE def,
						   char * proc));
extern G_MARKER_SIZE	g_check_marker_size G_ARGS ((G_MARKER_SIZE size,
						     G_MARKER_SIZE def,
						     char * proc));
extern G_MARKER_TYPE	g_check_marker_type G_ARGS ((G_MARKER_TYPE type,
						     G_MARKER_TYPE def,
						     char * proc));
extern G_FONT		g_check_font G_ARGS
			    ((G_FONT font, G_FONT def, char * proc));
extern void		g_set_line G_ARGS ((G_COLOR color, G_LINE_WIDTH width,
					    G_LINE_TYPE type));
extern void		g_set_area G_ARGS ((G_COLOR color));
extern void		g_set_scale G_ARGS
			    ((G_P_POS x_left, G_P_POS x_right,
			      G_P_POS y_bottom, G_P_POS y_top,
			      G_P_POS x_left_std, G_P_POS y_top_std,
			      G_P_DIM width_std, G_P_DIM height_std));
extern unsigned long	g_set_bw_color G_ARGS ((unsigned long mask));
extern char *		g_malloc G_ARGS ((unsigned size));
extern char *		g_realloc G_ARGS ((char * p, unsigned size));
extern void		g_warning G_ARGS ((char * proc, char * message));
extern void		g_error G_ARGS ((char * message));
extern void		g_check_init G_ARGS ((void));
extern void		g_msleep G_ARGS ((int msec));

extern void		Cg_area_color G_ARGS ((G_COLOR color));
extern void		Cg_bird_view G_ARGS
			    ((G_P_DIM x_wid, G_P_DIM y_wid, G_P_DIM z_wid,
			      G_P_REAL z_bottom, G_P_REAL z_top,
			      G_P_DIM distance, G_P_REAL theta, G_P_REAL phi,
			      G_P_POS x_left_std, G_P_POS y_top_std,
			      G_P_DIM x_wid_std, G_P_DIM y_wid_std,
			      G_REAL * data, int n_x, int n_y, int direction));
extern void		Cg_box G_ARGS ((G_P_POS x_left, G_P_POS x_right,
				        G_P_POS y_bottom, G_P_POS y_top,
				        G_BOOL edge, G_BOOL fill));
extern void		Cg_circle G_ARGS
			    ((G_P_POS x, G_P_POS y, G_P_DIM radius_x,
			      G_BOOL edge, G_BOOL fill));
extern void		Cg_cls G_ARGS ((void));
extern void		Cg_contln G_ARGS
			    ((G_P_POS x_left, G_P_POS x_right,
			      G_P_POS y_bottom, G_P_POS y_top,
			      G_REAL * data, int nx, int ny, G_P_POS height));
extern void		Cg_data_plot G_ARGS ((G_P_POS x_left, G_P_POS x_right,
				              G_POS y[], int n));
extern void		Cg_def_area G_ARGS ((int id, G_COLOR color));
extern void		Cg_def_line G_ARGS ((int id, G_COLOR color,
				             int width, G_LINE_TYPE type));
extern void		Cg_def_marker G_ARGS ((int id, G_COLOR color,
				               int size, G_MARKER_TYPE type));
extern void		Cg_def_scale G_ARGS
			    ((int id, G_P_POS x_left, G_P_POS x_right,
			      G_P_POS y_bottom, G_P_POS y_top,
			      G_P_POS x_left_std, G_P_POS y_top_std,
			      G_P_POS width_std, G_P_POS height_std));
extern void		Cg_def_text G_ARGS
			    ((int id, G_COLOR color, G_FONT font));
extern void		Cg_device G_ARGS ((G_DEV_MASK mask));
extern void		Cg_fake_bird_view G_ARGS
			    ((G_P_REAL z_bottom, G_P_REAL z_top,
			      G_P_REAL x_ratio, G_P_REAL y_ratio,
			      G_P_POS x_left_std, G_P_POS y_top_std,
			      G_P_DIM x_wid_std, G_P_DIM y_wid_std,
			      G_REAL * data, int n_x, int n_y, int direction));
extern void		Cg_fake_hidden G_ARGS
			    ((G_P_REAL z_bottom, G_P_REAL z_top,
			      G_P_REAL x_ratio, G_P_REAL y_ratio,
			      G_P_POS x_left_std, G_P_POS y_top_std,
			      G_P_DIM x_wid_std, G_P_DIM y_wid_std,
			      G_REAL * data, int n_x, int n_y, int direction,
			      int side_paint, int intv_x, int intv_y));
extern void		Cg_hidden G_ARGS
			    ((G_P_DIM x_wid, G_P_DIM y_wid, G_P_DIM z_wid,
			      G_P_REAL z_bottom, G_P_REAL z_top,
			      G_P_DIM distance, G_P_REAL theta, G_P_REAL phi,
			      G_P_POS x_left_std, G_P_POS y_top_std,
			      G_P_DIM x_wid_std, G_P_DIM y_wid_std,
			      G_REAL * data, int n_x, int n_y, int direction,
			      int side_paint, int intv_x, int intv_y));
extern void		Cg_init G_ARGS ((char * file_name, G_P_DIM win_width,
					 G_P_DIM win_height));
extern void		Cg_line_color G_ARGS ((G_COLOR color));
extern void		Cg_line_type G_ARGS ((G_LINE_TYPE type));
extern void		Cg_line_width G_ARGS ((G_LINE_WIDTH width));
extern void		Cg_marker G_ARGS ((G_P_POS x, G_P_POS y));
extern void		Cg_marker_color G_ARGS ((G_COLOR color));
extern void		Cg_marker_size G_ARGS ((G_MARKER_SIZE size));
extern void		Cg_marker_type G_ARGS ((G_MARKER_TYPE type));
extern void		Cg_move G_ARGS ((G_P_POS x, G_P_POS y));
extern void		Cg_plot G_ARGS ((G_P_POS x, G_P_POS y));
extern void		Cg_point G_ARGS ((G_P_POS x, G_P_POS y));
extern void		Cg_polygon G_ARGS
			    ((G_POS x[], G_POS y[], int n,
			      G_BOOL edge, G_BOOL fill));
extern void		Cg_polyline G_ARGS ((G_POS x[], G_POS y[], int n));
extern void		Cg_sel_area G_ARGS ((int id));
extern void		Cg_sel_line G_ARGS ((int id));
extern void		Cg_sel_marker G_ARGS ((int id));
extern void		Cg_sel_scale G_ARGS ((int id));
extern void		Cg_sel_text G_ARGS ((int id));
extern void		Cg_sleep G_ARGS ((G_P_REAL time));
extern void		Cg_term G_ARGS ((void));
extern void		Cg_text G_ARGS ((G_P_POS x, G_P_POS y, char * string));
extern void		Cg_text_color G_ARGS ((G_COLOR color));
extern void		Cg_text_font G_ARGS ((G_FONT font));

