
static char rcsid[] = "$Id: g_sformat.c,v 1.4 1995/09/27 03:48:38 nakano Exp nakano $";

/*
 *	sprintf interface for Fortran 77 by H.Nakano
 *
 *	RESTRICTION :	'%' must occur exactly once in format string
 *			'h' required for short integers (e.g. %hd)
 *			'*' not available for column length or precision
 */

#include "g_private.h"
#ifdef __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif

#if G_F77
#if G_F77_SIZE_BY_LONG
#if G_F77_SIZE_BY_REF
#define va_param_size(ap)	(* va_arg (ap, long *))
#else
#define va_param_size(ap)	(va_arg (ap, long))
#endif
#else
#if G_F77_SIZE_BY_REF
#define va_param_size(ap)	(* va_arg (ap, short *))
#else
#define va_param_size(ap)	(va_arg (ap, short))
#endif
#endif

typedef	enum
{ SF_CHAR, SF_SHORT, SF_INT, SF_LONG, SF_FLOAT, SF_DOUBLE, SF_STR }
SF_TYPE;

typedef union
{
    char	char_val;
    short	short_val;
    int		int_val;
    long	long_val;
    float	float_val;
    double	double_val;
    char *	str_val;
}
SF_VAL;

static void
do_sformat (buf, length, in)
    char *	buf;
    int *	length;
    va_list	in;
{
    char	format[G_MAX_TEXT_LENGTH + 1];
    char *	dst = format;
    char *	src;
    char *	str;
    int		buf_length, format_length, str_length;
    int		i;
    SF_VAL	val;
    SF_TYPE	type;

    src = va_arg (in, char *);

    for (i = 0, format_length = -1; i != format_length; i ++)
    {
	char	c;

	c = * (dst ++) = * (src ++);

	if (c == '%' && format_length < 0)
	{
	    G_BOOL	done;
	    SF_TYPE	int_type = SF_INT;
	    SF_TYPE	real_type = SF_FLOAT;

	    for (done = G_NO; ! done; i ++)
	    {
		c = * (dst ++) = * (src ++);

		switch (c)
		{
		    case 'l': case 'L':
			int_type = SF_LONG;
			real_type = SF_DOUBLE;
			break;
		    case 'h': case 'H':
			int_type = SF_SHORT;
			dst --;
			break;
		    case 'c': case 'C':
			int_type = SF_CHAR;
			/* Note : Never break. */
		    case 'd': case 'D':
		    case 'o': case 'O':
		    case 'x': case 'X':
		    case 'u': case 'U':
			switch (type = int_type)
			{
			    case SF_CHAR:
				val.char_val = * va_arg (in, char *);
				break;
			    case SF_SHORT:
				val.short_val = * va_arg (in, short *);
				break;
			    case SF_INT:
				val.int_val = * va_arg (in, int *);
				break;
			    case SF_LONG:
				val.long_val = * va_arg (in, long *);
				break;
			}
			done = G_YES;
			break;
		    case 'e': case 'E':
		    case 'f': case 'F':
		    case 'g': case 'G':
			switch (type = real_type)
			{
			    case SF_FLOAT:
				val.float_val = * va_arg (in, float *);
				break;
			    case SF_DOUBLE:
				val.double_val = * va_arg (in, double *);
				break;
			}
			done = G_YES;
			break;
		    case 's': case 'S':
			type = SF_STR;
			val.str_val= va_arg (in, char *);
			done = G_YES;
			break;
		    case '*':
			/* not implemented */
			return;
		    case '%':
			done = G_YES;
			break;
		}

		if (done && c != '%')
		{
		    buf_length = va_param_size (in);
#if G_F77_ALL_SIZE
		    va_param_size (in);	/* for the size of 'length' */
#endif
		    format_length = va_param_size (in);
		}
	    }
	}
    }

    * dst = '\0';

    switch (type)
    {
	case SF_CHAR:
	    sprintf (buf, format, val.char_val);
	    break;
	case SF_SHORT:
	    sprintf (buf, format, val.short_val);
	    break;
	case SF_INT:
	    sprintf (buf, format, val.int_val);
	    break;
	case SF_LONG:
	    sprintf (buf, format, val.long_val);
	    break;
	case SF_FLOAT:
	    sprintf (buf, format, val.float_val);
	    break;
	case SF_DOUBLE:
	    sprintf (buf, format, val.double_val);
	    break;
	case SF_STR:
	    str_length = va_param_size (in);
	    str = malloc ((unsigned int) (sizeof (char) * (str_length + 1)));

	    if (str)
	    {
		strncpy (str, val.str_val, str_length);
		str[str_length] = '\0';
		sprintf (buf, format, str);
		free (str);
	    }
	    break;
    }

    * length = strlen (buf);

    for (i = * length; i < buf_length; i ++)
	buf[i] = ' ';
}

void
#ifdef __STDC__
F77g_sformat (char * buf, int * length, ...)
#else
F77g_sformat (buf, length, va_alist)
    char *	buf;
    int *	length;
    va_dcl
#endif
{
    va_list	in;

#ifdef __STDC__
    va_start(in, length);
#else
    va_start(in);
#endif
    do_sformat (buf, length, in);
    va_end(in);
}

void
#ifdef __STDC__
g_sformat__ (char * buf, int * length, ...)
#else
g_sformat__ (buf, length, va_alist)
    char *	buf;
    int *	length;
    va_dcl
#endif
{
    va_list	in;

#ifdef __STDC__
    va_start(in, length);
#else
    va_start(in);
#endif
    do_sformat (buf, length, in);
    va_end(in);
}

#endif /* G_F77 */

