/* Draw_OSMesa2.c */
/* Reference osdemo.c */
/* Demo of off-screen Mesa rendering */
/*
 * See Mesa/include/GL/osmesa.h for documentation of the OSMesa functions.
 *
 * If you want to render BIG images you'll probably have to increase
 * MAX_WIDTH and MAX_HEIGHT in src/config.h.
 *
 * This program is in the public domain.
 *
 * Brian Paul
 *
 * PPM output provided by Joerg Schmalzl.
 */

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include "GL/osmesa.h"
#include "GL/glu.h"

#define PIXEL 400
#define MS 260

int Width = PIXEL;
int Height = PIXEL;

/* argument */
int Mgx,Mgy; 
int fn;
double dgx,dgy;
double *u,*v,*w;
double x_range,y_range,xy_range;

static void render_image(void)
{
  int i,j;
  double *p,*q,*r,*pp,*qq,*rr;

  p = u; q = v; r = w; 
  pp = u + (Mgx + 1); qq = v + (Mgx + 1); rr = w + (Mgx + 1);
  for (j=0;j<Mgy;j++){
    for (i=0;i<Mgx;i++){
      glBegin(GL_POLYGON);
      glColor3f(*p,*q,*r);
      glVertex2f(dgx*i,dgy*j);
      glColor3f(*(p+1), *(q+1), *(r+1));
      glVertex2f(dgx*(i+1),dgy*j);
      glColor3f(*(pp+1), *(qq+1), *(rr+1));
      glVertex2f(dgx*(i+1),dgy*(j+1));
      glColor3f(*pp,*qq,*rr);
      glVertex2f(dgx*i,dgy*(j+1));
      glEnd();
      p++;q++;r++;pp++;qq++;rr++;
    }
    p++;q++;r++;pp++;qq++;rr++;
  }
  glBegin(GL_LINE_LOOP);
  glColor3f(0.0,0.0,0.0);
  glVertex2f(0.0,0.0);
  glVertex2f(x_range,0.0);
  glVertex2f(x_range,y_range);
  glVertex2f(0.0,y_range);
  glEnd(); 
}

void set_view(){
  glClearColor(1.0, 1.0, 1.0, 1.0);
  glClear(GL_COLOR_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslatef(-0.5*x_range,-0.5*y_range,0.0);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(-0.5*x_range-0.1*xy_range,0.5*x_range+0.1*xy_range,
	     -0.5*y_range-0.1*xy_range,0.5*y_range+0.1*xy_range);
  glRectf(-0.1*x_range,-0.1*y_range,1.1*x_range,1.1*y_range);
}

void save_file(FILE *f, void *buffer){
  /* write PPM file */
  if (f) {
    int i, x, y;
    GLubyte *ptr;
    ptr = buffer;
    fprintf(f,"P6\n");
    fprintf(f,"# ppm-file created\n");
    fprintf(f,"%i %i\n", Width,Height);
    fprintf(f,"255\n");
    for (y=Height-1; y>=0; y--) {
      for (x=0; x<Width; x++) {
        i = (y*Width + x) * 4;
        fputc(ptr[i], f);   /* write red */
        fputc(ptr[i+1], f); /* write green */
        fputc(ptr[i+2], f); /* write blue */
      }
    }
    fclose(f);
  }
  printf("all done\n");
}

void change_image_format(char *in, char *out)
{
   char string[300];
   sprintf(string, "convert %s %s", in, out);
   system(string);
}

/* void draw_( int *p_fn, int *p_Mgx, int *p_Mgy, double *p_dgx, double *p_dgy,
	  double *p, double *q, double *r, char *image_file, char *image_dir)
*/
void draw_( int p_fn, int p_Mgx, int p_Mgy, double p_dgx, double p_dgy,
          double *p, double *q, double *r, char *image_file, char *image_dir) 

{
  char ppmfile[BUFSIZ],jpegfile[BUFSIZ];
  FILE *fp;
  OSMesaContext ctx;
  void *buffer;


  fn = p_fn; Mgx = p_Mgx; Mgy = p_Mgy; dgx = p_dgx; dgy = p_dgy;
  u = p; v = q; w = r;
  printf("%d %d %d %lf %lf\n", fn, Mgx, Mgy, dgx, dgy);

  x_range = dgx*Mgx;y_range=dgy*Mgy;
  xy_range = ((x_range > y_range) ? x_range : y_range);
  Width  = (int)((x_range / xy_range + 0.2) / 1.2 * PIXEL);
  Height = (int)((y_range / xy_range + 0.2) / 1.2 * PIXEL);
      
  /* Create an RGBA-mode context */
  ctx = OSMesaCreateContext( GL_RGBA, NULL );    
   
  /* Allocate the image buffer */
  buffer = malloc( Width * Height * 4 );

  /* Bind the buffer to the context and make it current */
  OSMesaMakeCurrent( ctx, buffer, GL_UNSIGNED_BYTE, Width, Height );

  set_view();  
  render_image();   
  glFlush();

  sprintf(ppmfile, "%s/%s%04d.ppm",image_dir,image_file,fn);
  sprintf(jpegfile,"%s/%s%04d.jpg",image_dir,image_file,fn);

  if ((fp = fopen(ppmfile, "w")) == NULL){
    perror(ppmfile);
    exit(1);
  }
  save_file(fp, buffer);   
  /*fclose(fp); */
  change_image_format(ppmfile,jpegfile);
  unlink(ppmfile);
  /* free the image buffer */
  free(buffer); 
  /* destroy the context */
  OSMesaDestroyContext(ctx);
/*  return 0; */
}
