/* ================================================================================ */
/* |                                                                              | */
/* |         FitzHugh-NagumoIC[@ŉvO                    | */
/* |                                                                              | */
/* |         x_{t}  =  ( x * ( 1-x ) * ( x-a ) - y ) /eps                         | */
/* |         y_{t}  =    x - gam * y                                              | */
/* |                                                                              | */
/* |                                       This program is in the public domain.  | */
/* |                                Programmed by M.Yadome. 2008/12/18, ver. 1.0  | */
/* |                                                                              | */
/* ================================================================================ */


/* ================================================================================ */
/*           wb_t@C                                                         */
/* ================================================================================ */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>


/* ================================================================================ */
/*           }N`                                                             */
/* ================================================================================ */
#define  DATADIR   "data"         /*  f[^fBNg                          */
#define  DIRMODE   0744           /*  f[^fBNg̋[h            */
#define  FILENAME  "ode2e.dat"    /*  f[^t@C                              */
#define  MAXSTEP   1500000        /*  ővZ                                  */
#define  SOUTPUT   3000           /*  SOUTPUTƂɃf[^o                       */
#define  SLEEP     0.01           /*  SLEEPb~                         */
#define  dt        0.000001       /*  ԍݕ                                    */
#define  a         0.2            /*  ̃p[^ݒ                        */
#define  gam       1.0            /*  ̃p[^ݒ                        */
#define  eps       0.001          /*  ̃p[^ݒ                        */


/* ================================================================================ */
/*           FitzHugh-Nagumo equation                                               */
/* ================================================================================ */
double F( double x, double y ){   return (x*(1.0-x)*(x-a)-y)/eps;  }
double G( double x, double y ){   return x-gam*y;                  }


/* ================================================================================ */
/*           f[^fBNg̍쐬                                               */
/* ================================================================================ */
void make_dir()
{
  char str[100];
  if(  mkdir(DATADIR,DIRMODE) != 0  ){
    sprintf( str, "rm -rf %s", DATADIR );
    system( str );
    mkdir( DATADIR, DIRMODE );
  }
}


/* ================================================================================ */
/*           t@C݃[hŊJ                                         */
/* ================================================================================ */
FILE *fopen_w( char filename[] )
{
  FILE *fp;
  if(  (fp=fopen(filename,"w")) == NULL  ){
    printf( "cannot open output file\n" );
    exit(1);
  }
  return fp;
}


/* ================================================================================ */
/*           f[^t@C̍쐬                                                 */
/* ================================================================================ */
FILE *output_filename()
{
  char filename[100];
  FILE *fp;
  sprintf( filename, "%s/%s", DATADIR, FILENAME );
  fp  =  fopen_w( filename );
  return fp;
}


/* ================================================================================ */
/*           gnuplot̏ݒ                                                      */
/* ================================================================================ */
FILE *gnuplot_ini( int select )
{
  FILE *gp=NULL;
  if(  select == 1  ){
    gp = popen( "gnuplot -geometry 400x400+40+50" , "w" );
    fprintf( gp, "set xrange[0:%lf] \n", MAXSTEP*dt );  
  }else if(  select == 2  )
    gp = popen( "gnuplot -geometry 400x400+450+50", "w" );
  return gp;
}


/* ================================================================================ */
/*           l̐ݒ                                                           */
/* ================================================================================ */
void initial_condition( double *x, double *y )
{
  char c;
  printf( "input initial data [Y/N]:" );
  scanf( "%c%*c", &c );
  if(  c == 'y'  ){
    printf( "x  =  " );  scanf( "%lf%*c", x );
    printf( "y  =  " );  scanf( "%lf%*c", y );
  }else{
    *x  =  0.6;
    *y  =  0.0;
  }
}


/* ================================================================================ */
/*           f[^̏o                                                           */
/* ================================================================================ */
void output( FILE *fp, double t, double x, double y )
{
  fprintf( fp, "%20.15lf %20.15lf %20.15lf\n", t, x, y );
  fflush( fp );
}


/* ================================================================================ */
/*           IC[@                                                             */
/* ================================================================================ */
void euler_method( double *xx, double *yy )
{
  double x, y;
  x    =  *xx;
  y    =  *yy;
  *xx  =  x+dt*F( x, y );
  *yy  =  y+dt*G( x, y );
}


/* ================================================================================ */
/*           gnuplot̏o                                                          */
/* ================================================================================ */
void gnuplot2d( FILE *fp2, FILE *fp3 )
{
  fprintf( fp2, "plot \"%s/%s\" u 1:2 w l, \"\" u 1:3 w l\n", DATADIR, FILENAME );
  fprintf( fp3, "plot \"%s/%s\" u 2:3 w l\n"                , DATADIR, FILENAME );
  fflush( fp2 );
  fflush( fp3 );
}


/* ================================================================================ */
/*           main֐                                                               */
/* ================================================================================ */
int main()                                    /*                                    */
{                                             /*                                    */
  int    step;                                /*                                    */
  FILE   *fp, *gp1, *gp2;                     /*                                    */
  double x, y;                                /*                                    */
  make_dir();                                 /*  f[^fBNg̍쐬          */
  step  =  0;                                 /*                                    */
  initial_condition( &x, &y );                /*  l̐ݒ                      */
  fp   =  output_filename();                  /*  f[^t@C̍쐬            */
  gp1  =  gnuplot_ini( 1 );                   /*  gnuplot̏ݒ                 */
  gp2  =  gnuplot_ini( 2 );                   /*  gnuplot̏ݒ                 */
  output( fp, step*dt, x, y );                /*  f[^̏o                      */
  gnuplot2d( gp1, gp2 );                      /*  gnuplot̏o                     */
  usleep( SLEEP*1.0E+06 );                    /*  SLEEPb~             */
  for( step = 1; step <= MAXSTEP; step++ ){   /*                                    */
    euler_method( &x, &y );                   /*  IC[@                        */
    if(  step%SOUTPUT == 0  ){                /*                                    */
      output( fp, step*dt, x, y );            /*  f[^̏o                      */
      gnuplot2d( gp1, gp2 );                  /*  gnuplot̏o                     */
      usleep( SLEEP*1.0E+06 );                /*  SLEEPb~             */
    }                                         /*                                    */
  }                                           /*                                    */
  printf( "please hit return key!!" );        /*                                    */
  getchar();                                  /*  W͂Pǂݍ      */
  fclose( fp  );                              /*  t@C                  */
  pclose( gp1 );                              /*  gnuplot                   */
  pclose( gp2 );                              /*  gnuplot                   */
  return 0;                                   /*  I                          */
}                                             /*                                    */
