/* ================================================================================ */
/* |                                                                              | */
/* |         Mz@ŉvO - dirichlet -                       | */
/* |                                                                              | */
/* |         u_{t}  =  du * u_{xx}                                                | */
/* |                                                                              | */
/* |                                       This program is in the public domain.  | */
/* |                                Programmed by M.Yadome. 2008/12/18, ver. 1.0  | */
/* |                                                                              | */
/* ================================================================================ */


/* ================================================================================ */
/*           wb_t@C                                                         */
/* ================================================================================ */
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>


/* ================================================================================ */
/*           }N`                                                             */
/* ================================================================================ */
#define  DATADIR  "data"                   /*  f[^fBNg                 */
#define  DIRMODE  0744                     /*  f[^fBNg̋[h   */
#define  MAXSTEP  40000                    /*  ővZ                         */
#define  SOUTPUT  100                      /*  SOUTPUTƂɃf[^o              */
#define  SLEEP    0.0                      /*  SLEEPb~                */
#define  du       1.0                      /*  gUW                             */
#define  Lx       80.0                     /*                                   */
#define  dt       0.001                    /*  ԍݕ                           */
#define  N        400                      /*  ԕ                           */
#define  BCL      0.0                      /*  E()                         */
#define  BCR      0.0                      /*  E(E)                         */
#define  dx       Lx/N
#define  rd       du*dt/(dx*dx)


/* ================================================================================ */
/*           f[^fBNg̍쐬                                               */
/* ================================================================================ */
void make_dir()
{
  char str[100];
  if(  mkdir(DATADIR,DIRMODE) != 0  ){
    sprintf( str, "rm -rf %s", DATADIR );
    system( str );
    mkdir( DATADIR, DIRMODE );
  }
}


/* ================================================================================ */
/*           l̐ݒ                                                           */
/* ================================================================================ */
void initial_condition( double u[] )
{
  int i;
  for( i = 0; i <= N; i++ )
    u[i]  =  ( sin(     i*M_PI/N) \
          +    sin( 2.0*i*M_PI/N) \
          +    sin( 7.0*i*M_PI/N) \
          +    sin(11.0*i*M_PI/N) ) / 4.0;
}


/* ================================================================================ */
/*                                                                              */
/* ================================================================================ */
int digit_number( int num )
{
  int dn;
  dn  =  1;
  while( (num=num/10)>0 ){   dn++;   }
  return dn;
}


/* ================================================================================ */
/*           t@C݃[hŊJ                                         */
/* ================================================================================ */
FILE *fopen_w( char filename[] )
{
  FILE *fp;
  if(  (fp=fopen(filename,"w")) == NULL  ){
    printf( "cannot open output file\n" );
    exit(1);
  }
  return fp;
}


/* ================================================================================ */
/*           f[^̏o                                                           */
/* ================================================================================ */
FILE *output_gnuplot_ini()
{
  char filename[100];
  FILE *fp;
  sprintf( filename, "%s/plot", DATADIR );
  fp  =  fopen_w( filename );
  fprintf( fp, "set xrange[0:%20.15lf]\n", Lx );
  fprintf( fp, "set yrange[-1.0:1.0]\n"       );  
  return fp;
}
void output_gnuplot( int number, FILE *fp )
{
  fprintf( fp, "!echo \"%0*d\"\n"         , digit_number(MAXSTEP/SOUTPUT), number );
  fprintf( fp, "plot \"%0*d\" u 2:3 w l\n", digit_number(MAXSTEP/SOUTPUT), number );
  fprintf( fp, "!sleep %lf\n"             , SLEEP                                 );
  fflush( fp );
}
void output_data( double time, double u[], FILE *fp )
{
  int i;
  for( i = 0; i <= N; i++ )
    fprintf( fp, "%20.15E %20.15E %20.15E\n", time, i*dx, u[i] );
  fflush( fp );
}
void output( int number, double time, double u[], FILE *fp_gnuplot )
{
  char filename[100];
  FILE *fp_data;
  sprintf( filename, "%s/%0*d", DATADIR, digit_number(MAXSTEP/SOUTPUT), number );
  printf( "%s\n", filename );
  fp_data  =  fopen_w( filename );
  output_data( time, u, fp_data );
  fclose( fp_data );
  output_gnuplot( number, fp_gnuplot );
}


/* ================================================================================ */
/*           gnuplot̏ݒƏo                                                */
/* ================================================================================ */
FILE *gnuplot_ini()
{
  FILE *fp;
  fp  =  popen( "gnuplot -geometry 400x400+40+50", "w" );
  fprintf( fp, "set xrange[0:%20.15lf]\n", Lx );
  fprintf( fp, "set yrange[-1.0:1.0]\n"       );  
  return fp;
}
void gnuplot( int number, FILE *gp )
{
  fprintf( gp, "plot \"%s/%0*d\" u 2:3 w l\n", DATADIR, digit_number(MAXSTEP/SOUTPUT), number );
  fflush( gp );
}


/* ================================================================================ */
/*           z@                                                                 */
/* ================================================================================ */
void explicit_dirichlet( double u[] )
{
  int    i;
  double new_u[N];
  for( i = 1; i <= N-1; i++ )   new_u[i] = u[i] + rd*(u[i-1]-2.0*u[i]+u[i+1]);
  for( i = 1; i <= N-1; i++ )       u[i] = new_u[i];
       i = 0;                       u[i] = BCL;
       i = N;                       u[i] = BCR;
}


/* ================================================================================ */
/*           main֐                                                               */
/* ================================================================================ */
int main()
{
  int    step;
  FILE   *fp, *gp;
  double u[N+1];
  make_dir();                                                /*  f[^fBNg̍쐬      */
  step  =  0;                                                /*                                */
  initial_condition( u );                                    /*  l̐ݒ                  */
  fp  =  output_gnuplot_ini();                               /*                                */
  output( step/SOUTPUT, step*dt, u, fp );                    /*  f[^̏o                  */
  gp  =  gnuplot_ini();                                      /*  gnuplot̏ݒ             */
  gnuplot( step/SOUTPUT, gp );                               /*  gnuplot̏o                 */
  usleep( SLEEP*1.0E+06 );                                   /*  SLEEPb~         */
  for( step = 1; step <= MAXSTEP; step++ ){                  /*                                */
    explicit_dirichlet( u );                                 /*  z@                        */
    if(  step%SOUTPUT == 0  ){                               /*                                */
      output ( step/SOUTPUT, step*dt, u, fp );               /*  f[^̏o                  */
      gnuplot( step/SOUTPUT, gp             );               /*  gnuplot̏o                 */
      usleep( SLEEP*1.0E+06 );                               /*  SLEEPb~         */
    }                                                        /*                                */
  }                                                          /*                                */
  printf( "please hit return key!!" );                       /*                                */
  fprintf( fp, "!echo -n \"please hit return key!!\"\n" );   /*                                */
  fprintf( fp, "pause -1"                               );   /*                                */
  getchar();                                                 /*  W͂Pǂݍ  */
  fclose( fp );                                              /*  t@C              */
  pclose( gp );                                              /*  gnuplot               */
  return 0;                                                  /*  I                      */
}
